package com.chelvc.framework.common.crypto;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import javax.crypto.Cipher;

import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 密码处理器工厂接口
 *
 * @author Woody
 * @date 2024/6/25
 */
public interface CipherFactory {
    /**
     * 日志处理器实例
     */
    Logger log = LoggerFactory.getLogger(CipherFactory.class);

    /**
     * 获取密码处理器
     *
     * @param mode 加解密模式
     * @return 密码处理器实例
     */
    Cipher getCipher(int mode);

    /**
     * 获取加密处理器
     *
     * @return 加密处理器
     */
    default Cipher getEncryptor() {
        return this.getCipher(Cipher.ENCRYPT_MODE);
    }

    /**
     * 获取解密处理器
     *
     * @return 解密处理器
     */
    default Cipher getDecrypter() {
        return this.getCipher(Cipher.DECRYPT_MODE);
    }

    /**
     * 数据加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    default String encrypt(String plaintext) {
        return this.encrypt(plaintext, false);
    }

    /**
     * 数据加密
     *
     * @param plaintext 明文
     * @param force     是否忽略异常
     * @return 密文
     */
    default String encrypt(String plaintext, boolean force) {
        return plaintext == null ? null : this.encrypt(this.getEncryptor(), plaintext, force);
    }

    /**
     * 数据加密
     *
     * @param cipher    密码处理器
     * @param plaintext 明文
     * @return 密文
     */
    default String encrypt(@NonNull Cipher cipher, String plaintext) {
        return CodecUtils.encrypt(cipher, plaintext);
    }

    /**
     * 数据加密
     *
     * @param cipher    密码处理器
     * @param plaintext 明文
     * @param force     是否忽略异常
     * @return 密文
     */
    default String encrypt(@NonNull Cipher cipher, String plaintext, boolean force) {
        if (plaintext == null) {
            return null;
        }
        try {
            return this.encrypt(cipher, plaintext);
        } catch (Exception e) {
            if (!force) {
                throw e;
            }
            log.warn("Plaintext encrypt failed: {}, {}", plaintext, e.getMessage());
        }
        return plaintext;
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文数组
     * @return 密文数组
     */
    default String[] encrypt(String... plaintexts) {
        return this.encrypt(plaintexts, false);
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文数组
     * @param force      是否忽略异常
     * @return 密文数组
     */
    default String[] encrypt(String[] plaintexts, boolean force) {
        if (ObjectUtils.isEmpty(plaintexts)) {
            return plaintexts;
        }
        Cipher cipher = this.getEncryptor();
        String[] ciphertexts = new String[plaintexts.length];
        for (int i = 0; i < plaintexts.length; i++) {
            ciphertexts[i] = this.encrypt(cipher, plaintexts[i], force);
        }
        return ciphertexts;
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文集合
     * @return 密文集合
     */
    default Set<String> encrypt(Set<String> plaintexts) {
        return this.encrypt(plaintexts, false);
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文集合
     * @param force      是否忽略异常
     * @return 密文集合
     */
    default Set<String> encrypt(Set<String> plaintexts, boolean force) {
        if (ObjectUtils.isEmpty(plaintexts)) {
            return plaintexts;
        }
        Cipher cipher = this.getEncryptor();
        return plaintexts.stream().map(plaintext -> this.encrypt(cipher, plaintext, force)).collect(Collectors.toSet());
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文列表
     * @return 密文列表
     */
    default List<String> encrypt(List<String> plaintexts) {
        return this.encrypt(plaintexts, false);
    }

    /**
     * 数据加密
     *
     * @param plaintexts 明文列表
     * @param force      是否忽略异常
     * @return 密文列表
     */
    default List<String> encrypt(List<String> plaintexts, boolean force) {
        if (ObjectUtils.isEmpty(plaintexts)) {
            return plaintexts;
        }
        Cipher cipher = this.getEncryptor();
        return plaintexts.stream().map(plaintext -> this.encrypt(cipher, plaintext, force))
                .collect(Collectors.toList());
    }

    /**
     * 数据解密
     *
     * @param ciphertext 密文
     * @return 明文
     */
    default String decrypt(String ciphertext) {
        return this.decrypt(ciphertext, false);
    }

    /**
     * 数据解密
     *
     * @param ciphertext 密文
     * @param force      是否忽略异常
     * @return 明文
     */
    default String decrypt(String ciphertext, boolean force) {
        return ciphertext == null ? null : this.decrypt(this.getDecrypter(), ciphertext, force);
    }

    /**
     * 数据解密
     *
     * @param cipher     密码处理器
     * @param ciphertext 密文
     * @return 明文
     */
    default String decrypt(@NonNull Cipher cipher, String ciphertext) {
        return CodecUtils.decrypt(cipher, ciphertext);
    }

    /**
     * 数据解密
     *
     * @param cipher     密码处理器
     * @param ciphertext 密文
     * @param force      是否忽略异常
     * @return 明文
     */
    default String decrypt(@NonNull Cipher cipher, String ciphertext, boolean force) {
        if (ciphertext == null) {
            return null;
        }
        try {
            return this.decrypt(cipher, ciphertext);
        } catch (Exception e) {
            if (!force) {
                throw e;
            }
            log.warn("Ciphertext decrypt failed: {}, {}", ciphertext, e.getMessage());
        }
        return ciphertext;
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文数组
     * @return 明文数组
     */
    default String[] decrypt(String... ciphertexts) {
        return this.decrypt(ciphertexts, false);
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文数组
     * @param force       是否忽略异常
     * @return 明文数组
     */
    default String[] decrypt(String[] ciphertexts, boolean force) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        Cipher cipher = this.getDecrypter();
        String[] plaintexts = new String[ciphertexts.length];
        for (int i = 0; i < ciphertexts.length; i++) {
            plaintexts[i] = this.decrypt(cipher, ciphertexts[i], force);
        }
        return plaintexts;
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文集合
     * @return 明文集合
     */
    default Set<String> decrypt(Set<String> ciphertexts) {
        return this.decrypt(ciphertexts, false);
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文集合
     * @param force       是否忽略异常
     * @return 明文集合
     */
    default Set<String> decrypt(Set<String> ciphertexts, boolean force) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        Cipher cipher = this.getDecrypter();
        return ciphertexts.stream().map(ciphertext -> this.decrypt(cipher, ciphertext, force))
                .collect(Collectors.toSet());
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文列表
     * @return 明文列表
     */
    default List<String> decrypt(List<String> ciphertexts) {
        return this.decrypt(ciphertexts, false);
    }

    /**
     * 数据解密
     *
     * @param ciphertexts 密文列表
     * @param force       是否忽略异常
     * @return 明文列表
     */
    default List<String> decrypt(List<String> ciphertexts, boolean force) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        Cipher cipher = this.getDecrypter();
        return ciphertexts.stream().map(ciphertext -> this.decrypt(cipher, ciphertext, force))
                .collect(Collectors.toList());
    }
}
