package com.chelvc.framework.common.model;

import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Supplier;

import com.chelvc.framework.common.util.AssertUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;

/**
 * 对象池
 *
 * @param <T> 对象类型
 * @author Woody
 * @date 2024/6/15
 */
public class Pool<T> {
    /**
     * 目标对象
     */
    private class Target {
        private final T object;
        private volatile long timestamp;

        private Target(T object, long timestamp) {
            this.object = object;
            this.timestamp = timestamp;
        }
    }

    private final long idle;
    private final Supplier<T> supplier;
    private final Lock lock = new ReentrantLock();
    private final Map<Long, Target> cache = Maps.newConcurrentMap();
    private volatile long cleaned = System.currentTimeMillis();

    public Pool(@NonNull Supplier<T> supplier) {
        this(60000, supplier);
    }

    public Pool(long idle, @NonNull Supplier<T> supplier) {
        AssertUtils.check(idle > 0, () -> "Pool idle must be greater than 0");
        this.idle = idle;
        this.supplier = supplier;
    }

    /**
     * 获取对象实例
     *
     * @return 对象实例
     */
    public T get() {
        // 从对象池获取对象实例
        long timestamp = System.currentTimeMillis();
        Target target = this.cache.computeIfAbsent(
                Thread.currentThread().getId(),
                id -> new Target(this.supplier.get(), timestamp)
        );

        // 更新对象访问时间
        target.timestamp = timestamp;

        // 清理空闲对象实例
        if (timestamp - this.cleaned > this.idle && this.lock.tryLock()) {
            try {
                this.cache.entrySet().removeIf(entry -> timestamp - entry.getValue().timestamp > this.idle);
                this.cleaned = timestamp;
            } finally {
                this.lock.unlock();
            }
        }

        // 返回对象实例
        return target.object;
    }
}
