package com.chelvc.framework.rocketmq.config;

import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import com.chelvc.framework.rocketmq.fallback.RocketMQFallback;
import com.chelvc.framework.rocketmq.fallback.RocketMQMemoryProducer;
import com.chelvc.framework.rocketmq.producer.RocketMQProducerWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.SessionCredentialsProvider;
import org.apache.rocketmq.client.apis.StaticSessionCredentialsProvider;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.TransactionChecker;
import org.apache.rocketmq.client.apis.producer.TransactionResolution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * RocketMQ配置类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RocketMQConfigurer {
    private final RocketMQProperties properties;

    @Bean
    @ConditionalOnMissingBean(ClientServiceProvider.class)
    public ClientServiceProvider provider() {
        return ClientServiceProvider.loadService();
    }

    @Bean
    @ConditionalOnMissingBean(ClientConfiguration.class)
    public ClientConfiguration configuration() {
        SessionCredentialsProvider sessionCredentialsProvider =
                new StaticSessionCredentialsProvider(this.properties.getAccessKey(), this.properties.getSecretKey());
        return ClientConfiguration.newBuilder().setEndpoints(this.properties.getEndpoints())
                .setCredentialProvider(sessionCredentialsProvider).build();
    }

    @Bean
    @ConditionalOnMissingBean(TransactionChecker.class)
    public TransactionChecker transactionChecker() {
        return message -> {
            String topic = RocketMQContextHolder.topic(message), body = RocketMQContextHolder.body(message);
            log.error("RocketMQ transaction checker is missing: {}, {}", topic, body);
            return TransactionResolution.UNKNOWN;
        };
    }

    @Bean
    @ConditionalOnMissingBean(RocketMQListenerRegistry.class)
    public RocketMQListenerRegistry listenerRegistry(ClientServiceProvider provider,
                                                     ClientConfiguration configuration) {
        return new RocketMQListenerRegistry(provider, configuration);
    }

    @Bean
    @ConditionalOnExpression("'${rocketmq.fallbacks}'.contains('MEMORY')")
    public RocketMQProducerWrapper memoryProducerWrapper() {
        return new RocketMQProducerWrapper() {
            private final int order = properties.getFallbacks().indexOf(RocketMQFallback.MEMORY);

            @Override
            public int getOrder() {
                return this.order;
            }

            @Override
            public Producer wrap(Producer producer) {
                return new RocketMQMemoryProducer(producer);
            }
        };
    }
}
