package com.chelvc.framework.rocketmq.context;

import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.rocketmq.config.RocketMQProperties;
import lombok.NonNull;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.message.MessageBuilder;
import org.apache.rocketmq.client.apis.message.MessageId;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.Transaction;

/**
 * RocketMQ上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class RocketMQContextHolder {
    /**
     * 消息生产者
     */
    private static volatile Producer PRODUCER;

    /**
     * 配置属性
     */
    private static volatile RocketMQProperties PROPERTIES;

    /**
     * 客户端服务提供者
     */
    private static volatile ClientServiceProvider SERVICE_PROVIDER;

    private RocketMQContextHolder() {
    }

    /**
     * 获取配置属性
     *
     * @return 配置属性
     */
    private static RocketMQProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (RocketMQProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(RocketMQProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取消息生产者
     *
     * @return 消息生产者实例
     */
    public static Producer getProducer() {
        if (PRODUCER == null) {
            synchronized (Producer.class) {
                if (PRODUCER == null) {
                    PRODUCER = ApplicationContextHolder.getBean(Producer.class);
                }
            }
        }
        return PRODUCER;
    }

    /**
     * 获取服务提供者
     *
     * @return 服务提供者实例
     */
    public static ClientServiceProvider getServiceProvider() {
        if (SERVICE_PROVIDER == null) {
            synchronized (ClientServiceProvider.class) {
                if (SERVICE_PROVIDER == null) {
                    SERVICE_PROVIDER = ApplicationContextHolder.getBean(ClientServiceProvider.class);
                }
            }
        }
        return SERVICE_PROVIDER;
    }

    /**
     * 获取消息主题
     *
     * @param message 消息实例
     * @return 消息主题
     */
    public static String topic(@NonNull Message message) {
        return topic(message, ':');
    }

    /**
     * 获取消息主题
     *
     * @param message 消息实例
     * @return 消息主题
     */
    public static String topic(@NonNull MessageView message) {
        return topic(message, ':');
    }

    /**
     * 获取消息主题
     *
     * @param message   消息实例
     * @param delimiter TOPIC与TAG分隔符
     * @return 消息主题
     */
    public static String topic(@NonNull Message message, char delimiter) {
        String namespace = getProperties().getNamespace();
        String topic = message.getTopic() + delimiter + message.getTag().orElse(null);
        return topic.startsWith(namespace) ? topic.substring(namespace.length()) : topic;
    }

    /**
     * 获取消息主题
     *
     * @param message   消息实例
     * @param delimiter TOPIC与TAG分隔符
     * @return 消息主题
     */
    public static String topic(@NonNull MessageView message, char delimiter) {
        String namespace = getProperties().getNamespace();
        String topic = message.getTopic() + delimiter + message.getTag().orElse(null);
        return topic.startsWith(namespace) ? topic.substring(namespace.length()) : topic;
    }

    /**
     * 获取消息体内容
     *
     * @param message 消息实例
     * @return 消息体内容
     */
    public static String body(@NonNull Message message) {
        return StandardCharsets.UTF_8.decode(message.getBody()).toString();
    }

    /**
     * 获取消息体内容
     *
     * @param message 消息实例
     * @return 消息体内容
     */
    public static String body(@NonNull MessageView message) {
        return StandardCharsets.UTF_8.decode(message.getBody()).toString();
    }

    /**
     * 环境隔离
     *
     * @param original 原始标记
     * @return 环境隔离标记
     */
    public static String isolate(@NonNull String original) {
        String namespace = getProperties().getNamespace();
        return StringUtils.isEmpty(namespace) ? original : (namespace + original);
    }

    /**
     * 消息序列化
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @return 消息对象实例
     */
    public static Message serialize(@NonNull String topic, @NonNull Object payload, String ordering) {
        byte[] body = JacksonUtils.serialize(payload).getBytes(StandardCharsets.UTF_8);
        String session = JacksonUtils.serialize(SessionContextHolder.getSession(false));
        MessageBuilder builder = getServiceProvider().newMessageBuilder().setBody(body);
        int delimiter = topic.indexOf(':');
        if (delimiter <= 0) {
            builder.setTopic(isolate(topic));
        } else {
            builder.setTopic(isolate(topic.substring(0, delimiter)));
            builder.setTag(topic.substring(delimiter + 1));
        }
        if (Objects.nonNull(session)) {
            builder.addProperty(Session.NAMING, session);
        }
        if (StringUtils.notEmpty(ordering)) {
            builder.setMessageGroup(ordering);
        }
        return builder.build();
    }

    /**
     * 消息反序列化
     *
     * @param message 消息视图
     * @param type    消息类型
     * @return 消息体实例
     */
    public static Object deserialize(@NonNull MessageView message, @NonNull Type type) {
        return JacksonUtils.deserialize(body(message), type);
    }

    /**
     * 消息反序列化
     *
     * @param message  消息视图
     * @param function 消息处理函数
     * @param <T>      消息类型
     * @return 消息处理结果
     */
    public static <T> T deserialize(@NonNull MessageView message, @NonNull Function<MessageView, T> function) {
        Session session = JacksonUtils.deserialize(message.getProperties().get(Session.NAMING), Session.class);
        SessionContextHolder.setSession(session);
        try {
            return function.apply(message);
        } finally {
            SessionContextHolder.clearSessionContext();
        }
    }

    /**
     * 消息反序列化
     *
     * @param message  消息视图
     * @param type     消息类型
     * @param function 消息处理函数
     * @param <T>      消息类型
     * @return 消息处理结果
     */
    public static <T> T deserialize(@NonNull MessageView message, @NonNull Type type,
                                    @NonNull Function<Object, T> function) {
        return deserialize(message, view -> function.apply(deserialize(message, type)));
    }

    /**
     * 消息反序列化
     *
     * @param messages 消息视图集合
     * @param type     消息类型
     * @param function 消息处理函数
     * @param <T>      消息类型
     * @return 消息列表
     */
    public static <T> List<T> deserialize(@NonNull Collection<MessageView> messages, @NonNull Type type,
                                          @NonNull BiFunction<Object, Session, T> function) {
        if (ObjectUtils.isEmpty(messages)) {
            return Collections.emptyList();
        }
        return messages.stream().map(message -> {
            Object payload = deserialize(message, type);
            Session session = JacksonUtils.deserialize(message.getProperties().get(Session.NAMING), Session.class);
            return function.apply(payload, session);
        }).filter(Objects::nonNull).collect(Collectors.toList());
    }

    /**
     * 发送消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     */
    public static void send(@NonNull String topic, @NonNull Object payload) {
        send(topic, payload, (String) null);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering) {
        Producer producer = getProducer();
        Message message = serialize(topic, payload, ordering);
        try {
            producer.send(message);
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发送异步消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload) {
        sendAsync(topic, payload, null);
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering) {
        Message message = serialize(topic, payload, ordering);
        getProducer().sendAsync(message);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param executor 本地事务执行方法
     * @param <T>      消息类型
     */
    public static <T> void send(@NonNull String topic, @NonNull T payload,
                                @NonNull Function<MessageId, Boolean> executor) {
        send(topic, payload, null, executor);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param executor 本地事务执行方法
     * @param <T>      消息类型
     */
    public static <T> void send(@NonNull String topic, @NonNull T payload, String ordering,
                                @NonNull Function<MessageId, Boolean> executor) {
        // 发送事务半消息
        MessageId id;
        Transaction transaction;
        Producer producer = getProducer();
        Message message = serialize(topic, payload, ordering);
        try {
            transaction = producer.beginTransaction();
            id = producer.send(message, transaction).getMessageId();
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }

        // 执行本地事务回调函数
        Boolean success = executor.apply(id);

        // 提交/回滚消息事务
        try {
            if (Boolean.TRUE.equals(success)) {
                transaction.commit();
            } else {
                transaction.rollback();
            }
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }
}
