package com.chelvc.framework.security.config;

import java.util.Set;

import com.chelvc.framework.base.interceptor.BufferedRequestInterceptor;
import com.chelvc.framework.security.annotation.Confidential;
import com.chelvc.framework.security.context.DefaultSecurityCryptoContext;
import com.chelvc.framework.security.context.SecurityCryptoContext;
import com.chelvc.framework.security.interceptor.MethodSecurityExpression;
import com.chelvc.framework.security.jackson.JacksonArrayDecrypter;
import com.chelvc.framework.security.jackson.JacksonArrayEncryptor;
import com.chelvc.framework.security.jackson.JacksonListDecrypter;
import com.chelvc.framework.security.jackson.JacksonListEncryptor;
import com.chelvc.framework.security.jackson.JacksonSetDecrypter;
import com.chelvc.framework.security.jackson.JacksonSetEncryptor;
import com.chelvc.framework.security.jackson.JacksonStringDecrypter;
import com.chelvc.framework.security.jackson.JacksonStringEncryptor;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.introspect.Annotated;
import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector;
import com.fasterxml.jackson.databind.type.CollectionType;
import lombok.RequiredArgsConstructor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.access.expression.method.DefaultMethodSecurityExpressionHandler;
import org.springframework.security.access.expression.method.MethodSecurityExpressionHandler;
import org.springframework.security.access.expression.method.MethodSecurityExpressionOperations;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.method.configuration.GlobalMethodSecurityConfiguration;
import org.springframework.security.core.Authentication;

/**
 * 安全配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@EnableGlobalMethodSecurity(prePostEnabled = true)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SecurityConfigurer extends GlobalMethodSecurityConfiguration implements ApplicationContextAware {
    private final SecurityProperties properties;

    @Override
    protected MethodSecurityExpressionHandler createExpressionHandler() {
        return new DefaultMethodSecurityExpressionHandler() {
            @Override
            protected MethodSecurityExpressionOperations createSecurityExpressionRoot(
                    Authentication authentication, MethodInvocation invocation) {
                MethodSecurityExpression root = new MethodSecurityExpression(authentication);
                root.setThis(invocation.getThis());
                root.setPermissionEvaluator(getPermissionEvaluator());
                root.setTrustResolver(getTrustResolver());
                root.setRoleHierarchy(getRoleHierarchy());
                root.setDefaultRolePrefix(getDefaultRolePrefix());
                return root;
            }
        };
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        // 设置数据加解密处理器
        ObjectMapper mapper = applicationContext.getBean(ObjectMapper.class);
        mapper.setAnnotationIntrospector(new JacksonAnnotationIntrospector() {
            @Override
            public Object findSerializer(Annotated am) {
                if (!am.hasAnnotation(Confidential.class)) {
                    return super.findSerializer(am);
                }

                JavaType type = am.getType();
                Class<?> raw = am.getRawType();
                if (raw == String.class) {
                    return JacksonStringEncryptor.class;
                } else if (raw.isArray() && raw.getComponentType() == String.class) {
                    return JacksonArrayEncryptor.class;
                } else if (type instanceof CollectionType && type.getContentType().getRawClass() == String.class) {
                    if (Set.class.isAssignableFrom(type.getRawClass())) {
                        return JacksonSetEncryptor.class;
                    }
                    return JacksonListEncryptor.class;
                }
                return super.findSerializer(am);
            }

            @Override
            public Object findDeserializer(Annotated am) {
                if (!am.hasAnnotation(Confidential.class)) {
                    return super.findDeserializer(am);
                }

                JavaType type = am.getType();
                Class<?> raw = am.getRawType();
                if (raw == String.class) {
                    return JacksonStringDecrypter.class;
                } else if (raw.isArray() && raw.getComponentType() == String.class) {
                    return JacksonArrayDecrypter.class;
                } else if (type instanceof CollectionType && type.getContentType().getRawClass() == String.class) {
                    if (Set.class.isAssignableFrom(type.getRawClass())) {
                        return JacksonSetDecrypter.class;
                    }
                    return JacksonListDecrypter.class;
                }
                return super.findDeserializer(am);
            }
        });
    }

    @Bean
    @ConditionalOnMissingBean(SecurityCryptoContext.class)
    public SecurityCryptoContext securityCryptoContext() {
        return new DefaultSecurityCryptoContext(this.properties);
    }

    @Bean
    @ConditionalOnMissingBean(BufferedRequestInterceptor.class)
    public BufferedRequestInterceptor bufferedRequestInterceptor() {
        return new BufferedRequestInterceptor();
    }
}
